@@ -19,15 +19,14 @@
 */
 
 #pragma once
-
+#include "kube_export.h"
 #include <QObject>
 #include <QString>
-#include <QStringList>
-#include <QStringListModel>
-#include <QStandardItemModel>
 #include <QVariant>
+#include <QStandardItemModel>
 #include <sink/applicationdomaintype.h>
 #include <KMime/Message>
+#include <mime/mailcrypto.h>
 
 #include "completer.h"
 #include "selector.h"
@@ -40,17 +39,23 @@ inline bool operator !=(const KMime::Types::Mailbox &l, const KMime::Types::Mail
 
 Q_DECLARE_METATYPE(KMime::Types::Mailbox);
 
+
 namespace KMime {
 class Message;
 }
 
-class ComposerController : public Kube::Controller
+class AddresseeModel;
+
+class KUBE_EXPORT ComposerController : public Kube::Controller
 {
     Q_OBJECT
 
     //Interface properties
     KUBE_CONTROLLER_PROPERTY(QString, Subject, subject)
     KUBE_CONTROLLER_PROPERTY(QString, Body, body)
+    KUBE_CONTROLLER_PROPERTY(bool, HtmlBody, htmlBody)
+    KUBE_CONTROLLER_PROPERTY(bool, Encrypt, encrypt)
+    KUBE_CONTROLLER_PROPERTY(bool, Sign, sign)
 
     //Set by identitySelector
     KUBE_CONTROLLER_PROPERTY(KMime::Types::Mailbox, Identity, identity)
@@ -60,68 +65,56 @@ class ComposerController : public Kube::Controller
     KUBE_CONTROLLER_PROPERTY(KMime::Message::Ptr, ExistingMessage, existingMessage)
     KUBE_CONTROLLER_PROPERTY(Sink::ApplicationDomain::Mail, ExistingMail, existingMail)
 
+    KUBE_CONTROLLER_PROPERTY(/*std::vector<Crypto::Key>*/QVariant, PersonalKeys, personalKeys)
+    KUBE_CONTROLLER_PROPERTY(bool, FoundPersonalKeys, foundPersonalKeys)
+
+    KUBE_CONTROLLER_LISTCONTROLLER(to)
+    KUBE_CONTROLLER_LISTCONTROLLER(cc)
+    KUBE_CONTROLLER_LISTCONTROLLER(bcc)
+    KUBE_CONTROLLER_LISTCONTROLLER(attachments)
+
     Q_PROPERTY (Completer* recipientCompleter READ recipientCompleter CONSTANT)
     Q_PROPERTY (Selector* identitySelector READ identitySelector CONSTANT)
-    //Q_PROPERTY (QValidator* subjectValidator READ subjectValidator CONSTANT)
-
-    Q_PROPERTY (QAbstractItemModel* toModel READ toModel CONSTANT)
-    Q_PROPERTY (QAbstractItemModel* ccModel READ ccModel CONSTANT)
-    Q_PROPERTY (QAbstractItemModel* bccModel READ bccModel CONSTANT)
-    Q_PROPERTY (QAbstractItemModel* attachmentModel READ attachmentModel CONSTANT)
 
     KUBE_CONTROLLER_ACTION(send)
     KUBE_CONTROLLER_ACTION(saveAsDraft)
 
 public:
+    enum LoadType {
+        Draft,
+        Reply,
+        Forward,
+    };
+    Q_ENUMS(LoadType);
+
     explicit ComposerController();
 
     Completer *recipientCompleter() const;
     Selector *identitySelector() const;
 
-    Q_INVOKABLE void loadMessage(const QVariant &draft, bool loadAsDraft);
+    Q_INVOKABLE void loadDraft(const QVariant &message);
+    Q_INVOKABLE void loadReply(const QVariant &message);
+    Q_INVOKABLE void loadForward(const QVariant &message);
 
-    QAbstractItemModel *toModel() const;
-    QAbstractItemModel *ccModel() const;
-    QAbstractItemModel *bccModel() const;
-    QAbstractItemModel *attachmentModel() const;
-
-    Q_INVOKABLE void addTo(const QString &);
-    Q_INVOKABLE void removeTo(const QString &);
-    Q_INVOKABLE void addCc(const QString &);
-    Q_INVOKABLE void removeCc(const QString &);
-    Q_INVOKABLE void addBcc(const QString &);
-    Q_INVOKABLE void removeBcc(const QString &);
-    Q_INVOKABLE void addAttachment(const QUrl &);
-    Q_INVOKABLE void removeAttachment(const QUrl &);
+signals:
+    void messageLoaded(const QString &body);
 
 public slots:
     virtual void clear() Q_DECL_OVERRIDE;
 
 private slots:
-    void updateSendAction();
-    void updateSaveAsDraftAction();
+    void findPersonalKey();
 
 private:
-    enum AttachmentRoles {
-        NameRole = Qt::UserRole + 1,
-        FilenameRole,
-        ContentRole,
-        MimeTypeRole,
-        DescriptionRole,
-        InlineRole,
-        IconNameRole,
-        UrlRole
-    };
+    void loadMessage(const QVariant &message, std::function<void(const KMime::Message::Ptr&)> callback);
 
     void recordForAutocompletion(const QByteArray &addrSpec, const QByteArray &displayName);
     void setMessage(const QSharedPointer<KMime::Message> &msg);
     void addAttachmentPart(KMime::Content *partToAttach);
     KMime::Message::Ptr assembleMessage();
+    std::vector<Crypto::Key> getRecipientKeys();
 
     QScopedPointer<Completer> mRecipientCompleter;
     QScopedPointer<Selector> mIdentitySelector;
-    QScopedPointer<QStringListModel> mToModel;
-    QScopedPointer<QStringListModel> mCcModel;
-    QScopedPointer<QStringListModel> mBccModel;
-    QScopedPointer<QStandardItemModel> mAttachmentModel;
+    bool mRemoveDraft = false;
 };
