@@ -21,8 +21,44 @@
 #include <sink/applicationdomaintype.h>
 #include <KContacts/VCardConverter>
 
+class MailsController : public Kube::ListPropertyController
+{
+public:
+
+    MailsController()
+        : Kube::ListPropertyController{{"email"}}
+    {
+    }
+
+    void set(const QStringList &list)
+    {
+        for (const auto &email: list) {
+            add({{"email", email}});
+        }
+    }
+};
+
+class PhonesController : public Kube::ListPropertyController
+{
+public:
+
+    PhonesController()
+        : Kube::ListPropertyController{{"number"}}
+    {
+    }
+
+    void set(const QStringList &list)
+    {
+        for (const auto &number: list) {
+            add({{"number", number}});
+        }
+    }
+};
+
 ContactController::ContactController()
     : Kube::Controller(),
+    controller_mails{new MailsController},
+    controller_phones(new PhonesController),
     action_save{new Kube::ControllerAction{this, &ContactController::save}}
 {
     updateSaveAction();
@@ -44,12 +80,13 @@ void ContactController::loadContact(const QVariant &contact)
         const auto &vcard = c->getVcard();
         KContacts::VCardConverter converter;
         const auto addressee = converter.parseVCard(vcard);
-        setEmails(addressee.emails());
+        static_cast<MailsController*>(mailsController())->set(addressee.emails());
+
         QStringList numbers;
         for (const auto &n : addressee.phoneNumbers()) {
             numbers << n.number();
         }
-        setPhoneNumbers(numbers);
+        static_cast<PhonesController*>(phonesController())->set(numbers);
 
         for(const auto &a :addressee.addresses()) {
             setStreet(a.street());
@@ -67,17 +104,3 @@ QVariant ContactController::contact() const
 {
     return QVariant{};
 }
-
-void ContactController::removeEmail(const QString &email)
-{
-    auto emails = getEmails();
-    emails.removeAll(email);
-    setEmails(emails);
-}
-
-void ContactController::addEmail(const QString &email)
-{
-    auto emails = getEmails();
-    emails.append(email);
-    setEmails(emails);
-}
