@@ -18,14 +18,15 @@
 
 
 #pragma once
-
+#include "kube_export.h"
 #include <QObject>
 #include <QString>
 #include <QStringList>
+#include <QStandardItemModel>
 
 #include "controller.h"
 
-class ContactController : public Kube::Controller
+class KUBE_EXPORT ContactController : public Kube::Controller
 {
     Q_OBJECT
 
@@ -34,8 +35,6 @@ class ContactController : public Kube::Controller
 
     //Interface properties
     KUBE_CONTROLLER_PROPERTY(QString, Name, name)
-    KUBE_CONTROLLER_PROPERTY(QStringList, Emails, emails)
-    KUBE_CONTROLLER_PROPERTY(QStringList, PhoneNumbers, phoneNumbers)
     KUBE_CONTROLLER_PROPERTY(QString, Street, street)
     KUBE_CONTROLLER_PROPERTY(QString, City, city)
     KUBE_CONTROLLER_PROPERTY(QString, Country, country)
@@ -43,14 +42,15 @@ class ContactController : public Kube::Controller
     KUBE_CONTROLLER_PROPERTY(QString, JobTitle, jobTitle)
     KUBE_CONTROLLER_PROPERTY(QByteArray, ImageData, imageData)
 
+    KUBE_CONTROLLER_LISTCONTROLLER(mails)
+    KUBE_CONTROLLER_LISTCONTROLLER(phones)
+
     KUBE_CONTROLLER_ACTION(save)
 
 public:
     explicit ContactController();
 
     Q_INVOKABLE void loadContact(const QVariant &contact);
-    Q_INVOKABLE void removeEmail(const QString &email);
-    Q_INVOKABLE void addEmail(const QString &email);
 
     QVariant contact() const;
 
