@@ -21,22 +21,29 @@
 #include "folderlistmodel.h"
 #include <sink/store.h>
 #include <sink/log.h>
+#include <sink/notifier.h>
+#include <sink/notification.h>
 #include <settings/settings.h>
 
 using namespace Sink;
 using namespace Sink::ApplicationDomain;
 
-FolderListModel::FolderListModel(QObject *parent) : QSortFilterProxyModel()
+FolderListModel::FolderListModel(QObject *parent) : KRecursiveFilterProxyModel(parent)
 {
     setDynamicSortFilter(true);
     sort(0, Qt::AscendingOrder);
 
-    Query query;
-    query.setFlags(Sink::Query::LiveQuery | Sink::Query::UpdateStatus);
-    query.request<Folder::Name>().request<Folder::Icon>().request<Folder::Parent>().request<Folder::SpecialPurpose>();
-    query.requestTree<Folder::Parent>();
-    query.setId("foldertree");
-    runQuery(query);
+    //Automatically fetch all folders, otherwise the recursive filtering does not work.
+    QObject::connect(this, &QSortFilterProxyModel::sourceModelChanged, [this] () {
+        if (sourceModel()) {
+            QObject::connect(sourceModel(), &QAbstractItemModel::rowsInserted, sourceModel(), [this] (QModelIndex parent, int first, int last) {
+                for (int row = first; row <= last; row++) {
+                    auto idx = sourceModel()->index(row, 0, parent);
+                    sourceModel()->fetchMore(idx);
+                }
+            });
+        }
+    });
 }
 
 FolderListModel::~FolderListModel()
@@ -54,6 +61,7 @@ QHash< int, QByteArray > FolderListModel::roleNames() const
     roles[DomainObject] = "domainObject";
     roles[Status] = "status";
     roles[Trash] = "trash";
+    roles[HasNewData] = "hasNewData";
 
     return roles;
 }
@@ -87,14 +95,46 @@ QVariant FolderListModel::data(const QModelIndex &idx, int role) const
                 return folder->getSpecialPurpose().contains(Sink::ApplicationDomain::SpecialPurpose::Mail::trash);
             }
             return false;
+        case HasNewData:
+            return mHasNewData.contains(folder->identifier());
     }
     return QSortFilterProxyModel::data(idx, role);
 }
 
+static QModelIndex findRecursive(QAbstractItemModel *model, const QModelIndex &parent, int role, const QVariant &value)
+{
+    for (auto row = 0; row < model->rowCount(parent); row++) {
+        const auto idx = model->index(row, 0, parent);
+        if (model->data(idx, role) == value) {
+            return idx;
+        }
+        auto result = findRecursive(model, idx, role, value);
+        if (result.isValid()) {
+            return result;
+        }
+    }
+    return {};
+}
+
 void FolderListModel::runQuery(const Query &query)
 {
     mModel = Store::loadModel<Folder>(query);
     setSourceModel(mModel.data());
+
+    Sink::Query resourceQuery;
+    resourceQuery.setFilter(query.getResourceFilter());
+    mNotifier.reset(new Sink::Notifier{resourceQuery});
+    mNotifier->registerHandler([&](const Sink::Notification &notification) {
+        if (notification.type == Sink::Notification::Info && notification.code == ApplicationDomain::NewContentAvailable) {
+            if (!notification.entities.isEmpty()) {
+                mHasNewData.insert(notification.entities.first());
+                auto idx = findRecursive(this, {}, Id, QVariant::fromValue(notification.entities.first()));
+                if (idx.isValid()) {
+                    emit dataChanged(idx, idx);
+                }
+            }
+        }
+    });
 }
 
 void FolderListModel::setAccountId(const QVariant &accountId)
@@ -108,6 +148,7 @@ void FolderListModel::setAccountId(const QVariant &accountId)
     query.request<Folder::Name>()
          .request<Folder::Icon>()
          .request<Folder::Parent>()
+         .request<Folder::Enabled>()
          .request<Folder::SpecialPurpose>();
     query.requestTree<Folder::Parent>();
     query.setId("foldertree" + account);
@@ -148,9 +189,30 @@ bool FolderListModel::lessThan(const QModelIndex &left, const QModelIndex &right
     return leftPriority < rightPriority;
 }
 
+bool FolderListModel::acceptRow(int sourceRow, const QModelIndex &sourceParent) const
+{
+    auto index = sourceModel()->index(sourceRow, 0, sourceParent);
+    Q_ASSERT(index.isValid());
+    const auto folder = index.data(Sink::Store::DomainObjectRole).value<Sink::ApplicationDomain::Folder::Ptr>();
+    Q_ASSERT(folder);
+    const auto enabled = folder->getEnabled();
+    return enabled;
+}
+
+void FolderListModel::fetchMore(const QModelIndex &parent)
+{
+    mHasNewData.remove(parent.data(Id).toByteArray());
+    QAbstractItemModel::fetchMore(parent);
+}
+
 void FolderListModel::setFolderId(const QVariant &folderId)
 {
     const auto folder = folderId.toString().toUtf8();
+    if (folder.isEmpty()) {
+        setSourceModel(nullptr);
+        mModel.clear();
+        return;
+    }
 
     //Get all folders of an account
     auto query = Query();
