@@ -19,17 +19,17 @@
 */
 
 #pragma once
-
-#include <QObject>
-#include <QSortFilterProxyModel>
+#include "kube_export.h"
+#include <krecursivefilterproxymodel.h>
 #include <QSharedPointer>
-#include <QStringList>
+#include <QSet>
+#include <sink/notifier.h>
 
 namespace Sink {
     class Query;
 }
 
-class FolderListModel : public QSortFilterProxyModel
+class KUBE_EXPORT FolderListModel : public KRecursiveFilterProxyModel
 {
     Q_OBJECT
 
@@ -56,7 +56,8 @@ public:
         Id,
         DomainObject,
         Status,
-        Trash
+        Trash,
+        HasNewData
     };
     Q_ENUMS(Roles)
 
@@ -69,8 +70,12 @@ public:
     QVariant folderId() const;
 protected:
     bool lessThan(const QModelIndex &left, const QModelIndex &right) const Q_DECL_OVERRIDE;
+    bool acceptRow(int sourceRow, const QModelIndex &sourceParent) const Q_DECL_OVERRIDE;
+    void fetchMore(const QModelIndex &left) Q_DECL_OVERRIDE;
 
 private:
     void runQuery(const Sink::Query &query);
     QSharedPointer<QAbstractItemModel> mModel;
+    QSet<QByteArray> mHasNewData;
+    QScopedPointer<Sink::Notifier> mNotifier;
 };
