@@ -23,7 +23,7 @@
 #include <sink/standardqueries.h>
 
 MailListModel::MailListModel(QObject *parent)
-    : QSortFilterProxyModel()
+    : QSortFilterProxyModel(parent)
 {
     setDynamicSortFilter(true);
     sort(0, Qt::DescendingOrder);
@@ -37,7 +37,34 @@ MailListModel::~MailListModel()
 
 void MailListModel::setFilter(const QString &filter)
 {
-    setFilterWildcard(filter);
+    if (filter.length() < 3 && !filter.isEmpty()) {
+        return;
+    }
+    auto oldQuery = mQuery;
+    auto query = mQuery;
+    if (!filter.isEmpty()) {
+        //Avoid live updates until we properly filter updates
+        query.setFlags(Sink::Query::NoFlags);
+        auto f = filter;
+        if (mCurrentQueryItem.isEmpty()) {
+            using namespace Sink::ApplicationDomain;
+            query.request<Mail::Subject>();
+            query.request<Mail::Sender>();
+            query.request<Mail::To>();
+            query.request<Mail::Cc>();
+            query.request<Mail::Bcc>();
+            query.request<Mail::Date>();
+            query.request<Mail::Unread>();
+            query.request<Mail::Important>();
+            query.request<Mail::Draft>();
+            query.request<Mail::Sent>();
+            query.request<Mail::Trash>();
+            query.request<Mail::Folder>();
+        }
+        query.filter({}, Sink::QueryBase::Comparator(f, Sink::QueryBase::Comparator::Fulltext));
+    }
+    runQuery(query);
+    mQuery = oldQuery;
 }
 
 QString MailListModel::filter() const
@@ -114,14 +141,14 @@ QVariant MailListModel::data(const QModelIndex &idx, int role) const
         case Date:
             return mail->getDate();
         case Unread:
-            if (mIsThreaded) {
-                return mail->getProperty("unreadCollected").toList().contains(true);
+            if (mail->isAggregate()) {
+                return mail->getCollectedProperty<Sink::ApplicationDomain::Mail::Unread>().contains(true);
             } else {
                 return mail->getUnread();
             }
         case Important:
-            if (mIsThreaded) {
-                return mail->getProperty("importantCollected").toList().contains(true);
+            if (mail->isAggregate()) {
+                return mail->getCollectedProperty<Sink::ApplicationDomain::Mail::Important>().contains(true);
             } else {
                 return mail->getImportant();
             }
@@ -141,11 +168,7 @@ QVariant MailListModel::data(const QModelIndex &idx, int role) const
             }
             return mail->getMimeMessage();
         case ThreadSize:
-            if (mIsThreaded) {
-                return mail->getProperty("count").toInt();
-            } else {
-                return 1;
-            }
+            return mail->count();
         case Mail:
             return QVariant::fromValue(mail);
         case Incomplete:
@@ -188,13 +211,15 @@ bool MailListModel::filterAcceptsRow(int sourceRow, const QModelIndex &sourcePar
 
 void MailListModel::runQuery(const Sink::Query &query)
 {
-    m_model = Sink::Store::loadModel<Sink::ApplicationDomain::Mail>(query);
-    setSourceModel(m_model.data());
-}
-
-bool MailListModel::isThreaded() const
-{
-    return mIsThreaded;
+    if (query.getBaseFilters().isEmpty() && query.ids().isEmpty()) {
+        mQuery = {};
+        m_model.clear();
+        setSourceModel(nullptr);
+    } else {
+        mQuery = query;
+        m_model = Sink::Store::loadModel<Sink::ApplicationDomain::Mail>(query);
+        setSourceModel(m_model.data());
+    }
 }
 
 void MailListModel::setParentFolder(const QVariant &parentFolder)
@@ -210,16 +235,14 @@ void MailListModel::setParentFolder(const QVariant &parentFolder)
         return;
     }
     mCurrentQueryItem = folder->identifier();
+    bool isThreaded = true;
     if (folder->getSpecialPurpose().contains(Sink::ApplicationDomain::SpecialPurpose::Mail::drafts) ||
         folder->getSpecialPurpose().contains(Sink::ApplicationDomain::SpecialPurpose::Mail::sent)) {
-        mIsThreaded = false;
-    } else {
-        mIsThreaded = true;
+        isThreaded = false;
     }
-    emit isThreadedChanged();
 
     Sink::Query query = [&] {
-        if (mIsThreaded) {
+        if (isThreaded) {
             return Sink::StandardQueries::threadLeaders(*folder);
         } else {
             Sink::Query query;
@@ -339,3 +362,85 @@ bool MailListModel::showDrafts() const
 {
     return false;
 }
+
+void MailListModel::setShowInbox(bool)
+{
+    using namespace Sink::ApplicationDomain;
+
+    Sink::Query folderQuery{};
+    folderQuery.containsFilter<Sink::ApplicationDomain::Folder::SpecialPurpose>(Sink::ApplicationDomain::SpecialPurpose::Mail::inbox);
+    folderQuery.request<Sink::ApplicationDomain::Folder::SpecialPurpose>();
+    folderQuery.request<Sink::ApplicationDomain::Folder::Name>();
+
+    Sink::Query query;
+    query.setFlags(Sink::Query::LiveQuery);
+    query.filter<Sink::ApplicationDomain::Mail::Folder>(folderQuery);
+    query.sort<Mail::Date>();
+    query.request<Mail::Subject>();
+    query.request<Mail::Sender>();
+    query.request<Mail::To>();
+    query.request<Mail::Cc>();
+    query.request<Mail::Bcc>();
+    query.request<Mail::Date>();
+    query.request<Mail::Unread>();
+    query.request<Mail::Important>();
+    query.request<Mail::Draft>();
+    query.request<Mail::Folder>();
+    query.request<Mail::Sent>();
+    query.request<Mail::Trash>();
+    query.request<Mail::MimeMessage>();
+    query.request<Mail::FullPayloadAvailable>();
+    mFetchMails = true;
+    mFetchedMails.clear();
+    qDebug() << "Running mail query for drafts: ";
+    //Latest mail at the top
+    sort(0, Qt::DescendingOrder);
+    runQuery(query);
+}
+
+bool MailListModel::showInbox() const
+{
+    return false;
+}
+
+void MailListModel::setEntityId(const QString &id)
+{
+    qDebug() << "Running mail query for mail with ID:" << id;
+    if (id.isEmpty()) {
+        mCurrentQueryItem.clear();
+        setSourceModel(nullptr);
+        return;
+    }
+    if (mCurrentQueryItem == id) {
+        return;
+    }
+    mCurrentQueryItem = id.toLatin1();
+    using namespace Sink::ApplicationDomain;
+    Sink::Query query;
+    query.setFlags(Sink::Query::LiveQuery);
+    query.filter(id.toUtf8());
+    query.request<Mail::Subject>();
+    query.request<Mail::Sender>();
+    query.request<Mail::To>();
+    query.request<Mail::Cc>();
+    query.request<Mail::Bcc>();
+    query.request<Mail::Date>();
+    query.request<Mail::Unread>();
+    query.request<Mail::Important>();
+    query.request<Mail::Draft>();
+    query.request<Mail::Folder>();
+    query.request<Mail::Sent>();
+    query.request<Mail::Trash>();
+    query.request<Mail::MimeMessage>();
+    query.request<Mail::FullPayloadAvailable>();
+    mFetchMails = true;
+    mFetchedMails.clear();
+    // Latest mail at the top
+    sort(0, Qt::DescendingOrder);
+    runQuery(query);
+}
+
+QString MailListModel::entityId() const
+{
+    return {};
+}
