@@ -19,21 +19,23 @@
 */
 
 #pragma once
-
+#include "kube_export.h"
 #include <sink/store.h>
 
 #include <QSortFilterProxyModel>
 #include <QSharedPointer>
 #include <QStringList>
 
-class MailListModel : public QSortFilterProxyModel
+class KUBE_EXPORT MailListModel : public QSortFilterProxyModel
 {
     Q_OBJECT
     Q_PROPERTY (QVariant parentFolder READ parentFolder WRITE setParentFolder)
     Q_PROPERTY (QVariant mail READ mail WRITE setMail)
     Q_PROPERTY (bool showDrafts READ showDrafts WRITE setShowDrafts)
+    Q_PROPERTY (bool showInbox READ showInbox WRITE setShowInbox)
+    Q_PROPERTY (QString entityId READ entityId WRITE setEntityId)
+
     Q_PROPERTY (QString filter READ filter WRITE setFilter)
-    Q_PROPERTY (bool isThreaded READ isThreaded NOTIFY isThreadedChanged)
 
 public:
     enum Status {
@@ -51,8 +53,6 @@ public:
     bool lessThan(const QModelIndex &left, const QModelIndex &right) const Q_DECL_OVERRIDE;
     bool filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const Q_DECL_OVERRIDE;
 
-    bool isThreaded() const;
-
     enum Roles {
         Subject  = Qt::UserRole + 1,
         Sender,
@@ -91,8 +91,11 @@ public:
     void setShowDrafts(bool);
     bool showDrafts() const;
 
-signals:
-    void isThreadedChanged();
+    void setShowInbox(bool);
+    bool showInbox() const;
+
+    void setEntityId(const QString &id);
+    QString entityId() const;
 
 private:
     void fetchMail(Sink::ApplicationDomain::Mail::Ptr mail);
@@ -101,5 +104,5 @@ private:
     bool mFetchMails = false;
     QSet<QByteArray> mFetchedMails;
     QByteArray mCurrentQueryItem;
-    bool mIsThreaded = true;
+    Sink::Query mQuery;
 };
