@@ -1,7 +1,8 @@
 /*
+    Copyright (c) 2009 Constantin Berzan <exit3219@gmail.com>
     Copyright (C) 2010 Klarälvdalens Datakonsult AB, a KDAB Group company, info@kdab.com
     Copyright (c) 2010 Leo Franchi <lfranchi@kde.org>
-    Copyright (c) 2016 Christian Mollekopf <mollekopf@kolabsys.com>
+    Copyright (c) 2017 Christian Mollekopf <mollekopf@kolabsys.com>
 
     This library is free software; you can redistribute it and/or modify it
     under the terms of the GNU Library General Public License as published by
@@ -29,6 +30,7 @@
 #include <QWebEngineSettings>
 #include <QWebEngineScript>
 #include <QSysInfo>
+#include <QUuid>
 #include <QTextCodec>
 #include <QTextDocument>
 
@@ -37,6 +39,8 @@
 
 #include <mimetreeparser/objecttreeparser.h>
 
+#include "mailcrypto.h"
+
 namespace KMime {
     namespace Types {
 static bool operator==(const KMime::Types::AddrSpec &left, const KMime::Types::AddrSpec &right)
@@ -49,6 +53,18 @@ static bool operator==(const KMime::Types::Mailbox &left, const KMime::Types::Ma
     return (left.addrSpec().asString() == right.addrSpec().asString());
 }
     }
+
+    Message* contentToMessage(Content* content) {
+        content->assemble();
+        const auto encoded = content->encodedContent();
+
+        auto message = new Message();
+        message->setContent(encoded);
+        message->parse();
+
+        return message;
+    }
+
 }
 
 static KMime::Types::Mailbox::List stripMyAddressesFromAddressList(const KMime::Types::Mailbox::List &list, const KMime::Types::AddrSpecList me)
@@ -65,16 +81,11 @@ static KMime::Types::Mailbox::List stripMyAddressesFromAddressList(const KMime::
     return addresses;
 }
 
-void initHeader(const KMime::Message::Ptr &message)
+static QString toPlainText(const QString &s)
 {
-    message->removeHeader<KMime::Headers::To>();
-    message->removeHeader<KMime::Headers::Subject>();
-    message->date()->setDateTime(QDateTime::currentDateTime());
-
-    const QStringList extraInfo = QStringList() << QSysInfo::prettyProductName();
-    message->userAgent()->fromUnicodeString(QString("%1/%2(%3)").arg(QString::fromLocal8Bit("Kube")).arg("0.1").arg(extraInfo.join(",")), "utf-8");
-    // This will allow to change Content-Type:
-    message->contentType()->setMimeType("text/plain");
+    QTextDocument doc;
+    doc.setHtml(s);
+    return doc.toPlainText();
 }
 
 QString replacePrefixes(const QString &str, const QStringList &prefixRegExps, const QString &newPrefix)
@@ -98,7 +109,8 @@ QString replacePrefixes(const QString &str, const QStringList &prefixRegExps, co
     if (rx.indexIn(tmp) == 0) {
         return tmp.replace(0, rx.matchedLength(), newPrefix + QLatin1String(" "));
     }
-    return str;
+    //No match, we just prefix the newPrefix
+    return newPrefix + " " + str;
 }
 
 const QStringList getForwardPrefixes()
@@ -188,9 +200,9 @@ QByteArray getRefStr(const KMime::Message::Ptr &msg)
     return retRefStr;
 }
 
-KMime::Content *createPlainPartContent(const KMime::Message::Ptr &msg, const QString &plainBody)
+KMime::Content *createPlainPartContent(const QString &plainBody, KMime::Content *parent = nullptr)
 {
-    KMime::Content *textPart = new KMime::Content(msg.data());
+    KMime::Content *textPart = new KMime::Content(parent);
     textPart->contentType()->setMimeType("text/plain");
     //FIXME This is supposed to select a charset out of the available charsets that contains all necessary characters to render the text
     // QTextCodec *charset = selectCharset(m_charsets, plainBody);
@@ -201,22 +213,21 @@ KMime::Content *createPlainPartContent(const KMime::Message::Ptr &msg, const QSt
     return textPart;
 }
 
-KMime::Content *createMultipartAlternativeContent(const KMime::Message::Ptr &msg, const QString &plainBody, const QString &htmlBody)
+KMime::Content *createMultipartAlternativeContent(const QString &plainBody, const QString &htmlBody, KMime::Message *parent = nullptr)
 {
-    KMime::Content *multipartAlternative = new KMime::Content(msg.data());
+    KMime::Content *multipartAlternative = new KMime::Content(parent);
     multipartAlternative->contentType()->setMimeType("multipart/alternative");
-    const QByteArray boundary = KMime::multiPartBoundary();
-    multipartAlternative->contentType()->setBoundary(boundary);
+    multipartAlternative->contentType()->setBoundary(KMime::multiPartBoundary());
 
-    KMime::Content *textPart = createPlainPartContent(msg, plainBody);
+    KMime::Content *textPart = createPlainPartContent(plainBody, multipartAlternative);
     multipartAlternative->addContent(textPart);
 
-    KMime::Content *htmlPart = new KMime::Content(msg.data());
+    KMime::Content *htmlPart = new KMime::Content(multipartAlternative);
     htmlPart->contentType()->setMimeType("text/html");
     //FIXME This is supposed to select a charset out of the available charsets that contains all necessary characters to render the text
     // QTextCodec *charset = selectCharset(m_charsets, htmlBody);
     // htmlPart->contentType()->setCharset(charset->name());
-    textPart->contentType()->setCharset("utf-8");
+    htmlPart->contentType()->setCharset("utf-8");
     htmlPart->contentTransferEncoding()->setEncoding(KMime::Headers::CE8Bit);
     htmlPart->fromUnicodeString(htmlBody);
     multipartAlternative->addContent(htmlPart);
@@ -224,52 +235,17 @@ KMime::Content *createMultipartAlternativeContent(const KMime::Message::Ptr &msg
     return multipartAlternative;
 }
 
-void addProcessedBodyToMessage(const KMime::Message::Ptr &msg, const QString &plainBody, const QString &htmlBody, bool forward)
+KMime::Content *createMultipartMixedContent(QVector<KMime::Content *> contents)
 {
-    //FIXME
-    // MessageCore::ImageCollector ic;
-    // ic.collectImagesFrom(mOrigMsg.data());
+    KMime::Content *multiPartMixed = new KMime::Content();
+    multiPartMixed->contentType()->setMimeType("multipart/mixed");
+    multiPartMixed->contentType()->setBoundary(KMime::multiPartBoundary());
 
-    // Now, delete the old content and set the new content, which
-    // is either only the new text or the new text with some attachments.
-    auto parts = msg->contents();
-    foreach (KMime::Content *content, parts) {
-        msg->removeContent(content, true/*delete*/);
+    for (const auto &content : contents) {
+        multiPartMixed->addContent(content);
     }
 
-    msg->contentType()->clear(); // to get rid of old boundary
-
-    const QByteArray boundary = KMime::multiPartBoundary();
-    KMime::Content *const mainTextPart =
-        htmlBody.isEmpty() ?
-        createPlainPartContent(msg, plainBody) :
-        createMultipartAlternativeContent(msg, plainBody, htmlBody);
-    mainTextPart->assemble();
-
-    KMime::Content *textPart = mainTextPart;
-    // if (!ic.images().empty()) {
-    //     textPart = createMultipartRelated(ic, mainTextPart);
-    //     textPart->assemble();
-    // }
-
-    // If we have some attachments, create a multipart/mixed mail and
-    // add the normal body as well as the attachments
-    KMime::Content *mainPart = textPart;
-    //FIXME
-    // if (forward) {
-    //     auto attachments = mOrigMsg->attachments();
-    //     attachments += mOtp->nodeHelper()->attachmentsOfExtraContents();
-    //     if (!attachments.isEmpty()) {
-    //         mainPart = createMultipartMixed(attachments, textPart);
-    //         mainPart->assemble();
-    //     }
-    // }
-
-    msg->setBody(mainPart->encodedBody());
-    msg->setHeader(mainPart->contentType());
-    msg->setHeader(mainPart->contentTransferEncoding());
-    msg->assemble();
-    msg->parse();
+    return multiPartMixed;
 }
 
 QString plainToHtml(const QString &body)
@@ -381,7 +357,6 @@ void setupPage(QWebEnginePage *page)
         page->settings()->setAttribute(QWebEngineSettings::WebGLEnabled, false);
         page->settings()->setAttribute(QWebEngineSettings::AutoLoadIconsForPage, false);
         page->settings()->setAttribute(QWebEngineSettings::Accelerated2dCanvasEnabled, false);
-        page->settings()->setAttribute(QWebEngineSettings::WebGLEnabled, false);
 
 #if QT_VERSION >= QT_VERSION_CHECK(5, 8, 0)
         page->settings()->setAttribute(QWebEngineSettings::FocusOnNavigationEnabled, false);
@@ -393,15 +368,7 @@ void plainMessageText(const QString &plainTextContent, const QString &htmlConten
 {
     QString result = plainTextContent;
     if (plainTextContent.isEmpty()) {   //HTML-only mails
-        auto page = new QWebEnginePage;
-        setupPage(page);
-        page->setHtml(htmlContent);
-        QObject::connect(page, &QWebEnginePage::loadFinished, [=] (bool ok) {
-            page->toPlainText([=] (const QString &plaintext) {
-                page->deleteLater();
-                callback(plaintext);
-            });
-        });
+        callback(toPlainText(htmlContent));
         return;
     }
 
@@ -621,7 +588,7 @@ static Recipients getRecipients(const KMime::Message::Ptr &origMsg, const KMime:
     KMime::Types::Mailbox::List toList;
     KMime::Types::Mailbox::List ccList;
     //FIXME
-    const ReplyStrategy replyStrategy = ReplySmart;
+    const ReplyStrategy replyStrategy = ReplyAll;
     switch (replyStrategy) {
     case ReplySmart: {
         if (auto hdr = origMsg->headerByType("Mail-Followup-To")) {
@@ -792,18 +759,16 @@ static Recipients getRecipients(const KMime::Message::Ptr &origMsg, const KMime:
     return {toList, ccList};
 }
 
-void MailTemplates::reply(const KMime::Message::Ptr &origMsg, const std::function<void(const KMime::Message::Ptr &result)> &callback)
+void MailTemplates::reply(const KMime::Message::Ptr &origMsg, const std::function<void(const KMime::Message::Ptr &result)> &callback, const KMime::Types::AddrSpecList &me)
 {
     //FIXME
     const bool alwaysPlain = true;
     KMime::Message::Ptr msg(new KMime::Message);
-    //FIXME
-    //Personal email addresses, required to detect the case where we reply to a message we have sent ourselves
-    KMime::Types::AddrSpecList me;
 
-    // const uint originalIdentity = identityUoid(origMsg);
-    initHeader(msg);
 
+    msg->removeHeader<KMime::Headers::To>();
+    msg->removeHeader<KMime::Headers::Subject>();
+    msg->contentType(true)->setMimeType("text/plain");
     msg->contentType()->setCharset("utf-8");
 
     const auto recipients = getRecipients(origMsg, me);
@@ -847,32 +812,115 @@ void MailTemplates::reply(const KMime::Message::Ptr &origMsg, const std::functio
     const auto htmlContent = otp.htmlContent();
 
     plainMessageText(plainTextContent, htmlContent, stripSignature, [=] (const QString &body) {
-        //Quoted body */
+        //Quoted body
         QString plainQuote = quotedPlainText(body, origMsg->from()->displayString());
         if (plainQuote.endsWith(QLatin1Char('\n'))) {
             plainQuote.chop(1);
         }
-        //The plain body is complete */
+        //The plain body is complete
         auto plainBodyResult = plainBody + plainQuote;
         htmlMessageText(plainTextContent, htmlContent, stripSignature, [=] (const QString &body, const QString &headElement) {
             //The html body is complete
-            auto htmlBodyResult = htmlBody + quotedHtmlText(body);
-            if (alwaysPlain) {
-                htmlBodyResult.clear();
-            } else {
-                makeValidHtml(htmlBodyResult, headElement);
-            }
+            const auto htmlBodyResult = [&]() {
+                if (!alwaysPlain) {
+                    auto htmlBodyResult = htmlBody + quotedHtmlText(body);
+                    makeValidHtml(htmlBodyResult, headElement);
+                    return htmlBodyResult;
+                }
+                return QString{};
+            }();
 
             //Assemble the message
-            addProcessedBodyToMessage(msg, plainBodyResult, htmlBodyResult, false);
-            applyCharset(msg, origMsg);
+            msg->contentType()->clear(); // to get rid of old boundary
+
+            KMime::Content *const mainTextPart =
+                htmlBodyResult.isEmpty() ?
+                createPlainPartContent(plainBodyResult, msg.data()) :
+                createMultipartAlternativeContent(plainBodyResult, htmlBodyResult, msg.data());
+            mainTextPart->assemble();
+
+            msg->setBody(mainTextPart->encodedBody());
+            msg->setHeader(mainTextPart->contentType());
+            msg->setHeader(mainTextPart->contentTransferEncoding());
+            //FIXME this does more harm than good right now.
+            // applyCharset(msg, origMsg);
             msg->assemble();
-            //We're done
+
             callback(msg);
         });
     });
 }
 
+void MailTemplates::forward(const KMime::Message::Ptr &origMsg,
+    const std::function<void(const KMime::Message::Ptr &result)> &callback)
+{
+    KMime::Message::Ptr wrapperMsg(new KMime::Message);
+
+    wrapperMsg->to()->clear();
+    wrapperMsg->cc()->clear();
+
+    // Decrypt the original message, it will be encrypted again in the composer
+    // for the right recipient
+    KMime::Message::Ptr forwardedMessage(new KMime::Message());
+    if (isEncrypted(origMsg.data())) {
+        qDebug() << "Original message was encrypted, decrypting it";
+        MimeTreeParser::ObjectTreeParser otp;
+        otp.parseObjectTree(origMsg.data());
+        otp.decryptParts();
+
+        auto htmlContent = otp.htmlContent();
+
+        KMime::Content *recreatedMsg =
+            htmlContent.isEmpty() ? createPlainPartContent(otp.plainTextContent()) :
+                                    createMultipartAlternativeContent(otp.plainTextContent(), htmlContent);
+
+        KMime::Message::Ptr tmpForwardedMessage;
+        auto attachments = otp.collectAttachmentParts();
+        if (!attachments.isEmpty()) {
+            QVector<KMime::Content *> contents = {recreatedMsg};
+            for (const auto &attachment : attachments) {
+                contents.append(attachment->node());
+            }
+
+            auto msg = createMultipartMixedContent(contents);
+
+            tmpForwardedMessage.reset(KMime::contentToMessage(msg));
+        } else {
+            tmpForwardedMessage.reset(KMime::contentToMessage(recreatedMsg));
+        }
+
+        origMsg->contentType()->fromUnicodeString(tmpForwardedMessage->contentType()->asUnicodeString(), "utf-8");
+        origMsg->assemble();
+        forwardedMessage->setHead(origMsg->head());
+        forwardedMessage->setBody(tmpForwardedMessage->encodedBody());
+        forwardedMessage->parse();
+
+    } else {
+        qDebug() << "Original message was not encrypted, using it as-is";
+        forwardedMessage = origMsg;
+    }
+
+    wrapperMsg->subject()->fromUnicodeString(
+        forwardSubject(forwardedMessage->subject()->asUnicodeString()), "utf-8");
+
+    const QByteArray refStr = getRefStr(forwardedMessage);
+    if (!refStr.isEmpty()) {
+        wrapperMsg->references()->fromUnicodeString(QString::fromLocal8Bit(refStr), "utf-8");
+    }
+
+    KMime::Content *fwdAttachment = new KMime::Content;
+
+    fwdAttachment->contentDisposition()->setDisposition(KMime::Headers::CDinline);
+    fwdAttachment->contentType()->setMimeType("message/rfc822");
+    fwdAttachment->contentDisposition()->setFilename(forwardedMessage->subject()->asUnicodeString() + ".eml");
+    fwdAttachment->setBody(KMime::CRLFtoLF(forwardedMessage->encodedContent(false)));
+
+    wrapperMsg->addContent(fwdAttachment);
+    wrapperMsg->assemble();
+
+    callback(wrapperMsg);
+}
+
 QString MailTemplates::plaintextContent(const KMime::Message::Ptr &msg)
 {
     MimeTreeParser::ObjectTreeParser otp;
@@ -880,9 +928,145 @@ QString MailTemplates::plaintextContent(const KMime::Message::Ptr &msg)
     const auto plain = otp.plainTextContent();
     if (plain.isEmpty()) {
         //Maybe not as good as the webengine version, but works at least for simple html content
-        QTextDocument doc;
-        doc.setHtml(otp.htmlContent());
-        return doc.toPlainText();
+        return toPlainText(otp.htmlContent());
     }
     return plain;
 }
+
+QString MailTemplates::body(const KMime::Message::Ptr &msg, bool &isHtml)
+{
+    MimeTreeParser::ObjectTreeParser otp;
+    otp.parseObjectTree(msg.data());
+    const auto html = otp.htmlContent();
+    if (html.isEmpty()) {
+        isHtml = false;
+        return otp.plainTextContent();
+    }
+    isHtml = true;
+    return html;
+}
+
+static KMime::Content *createAttachmentPart(const QByteArray &content, const QString &filename, bool isInline, const QByteArray &mimeType, const QString &name)
+{
+
+    KMime::Content *part = new KMime::Content;
+    part->contentDisposition(true)->setFilename(filename);
+    if (isInline) {
+        part->contentDisposition(true)->setDisposition(KMime::Headers::CDinline);
+    } else {
+        part->contentDisposition(true)->setDisposition(KMime::Headers::CDattachment);
+    }
+
+    part->contentType(true)->setMimeType(mimeType);
+    part->contentType(true)->setName(name, "utf-8");
+    // Just always encode attachments base64 so it's safe for binary data,
+    // except when it's another message
+    if(mimeType != "message/rfc822") {
+        part->contentTransferEncoding(true)->setEncoding(KMime::Headers::CEbase64);
+    }
+    part->setBody(content);
+    return part;
+}
+
+static KMime::Content *createBodyPart(const QString &body, bool htmlBody) {
+    if (htmlBody) {
+        return createMultipartAlternativeContent(toPlainText(body), body);
+    }
+    return createPlainPartContent(body);
+}
+
+static KMime::Types::Mailbox::List stringListToMailboxes(const QStringList &list)
+{
+    KMime::Types::Mailbox::List mailboxes;
+    for (const auto &s : list) {
+        KMime::Types::Mailbox mb;
+        mb.fromUnicodeString(s);
+        mailboxes << mb;
+    }
+    return mailboxes;
+}
+
+KMime::Message::Ptr MailTemplates::createMessage(KMime::Message::Ptr existingMessage,
+    const QStringList &to, const QStringList &cc, const QStringList &bcc,
+    const KMime::Types::Mailbox &from, const QString &subject, const QString &body, bool htmlBody,
+    const QList<Attachment> &attachments, const std::vector<Crypto::Key> &signingKeys,
+    const std::vector<Crypto::Key> &encryptionKeys, const Crypto::Key &attachedKey)
+{
+    auto mail = existingMessage;
+    if (!mail) {
+        mail = KMime::Message::Ptr::create();
+    } else {
+        //Content type is part of the body part we're creating
+        mail->removeHeader<KMime::Headers::ContentType>();
+        mail->removeHeader<KMime::Headers::ContentTransferEncoding>();
+    }
+
+    mail->date()->setDateTime(QDateTime::currentDateTime());
+    mail->userAgent()->fromUnicodeString(QString("%1/%2(%3)").arg(QString::fromLocal8Bit("Kube")).arg("0.1").arg(QSysInfo::prettyProductName()), "utf-8");
+
+    mail->to(true)->clear();
+    for (const auto &mb : stringListToMailboxes(to)) {
+        mail->to()->addAddress(mb);
+    }
+    mail->cc(true)->clear();
+    for (const auto &mb : stringListToMailboxes(cc)) {
+        mail->cc()->addAddress(mb);
+    }
+    mail->bcc(true)->clear();
+    for (const auto &mb : stringListToMailboxes(bcc)) {
+        mail->bcc()->addAddress(mb);
+    }
+
+    mail->from(true)->clear();
+    mail->from(true)->addAddress(from);
+
+    mail->subject(true)->fromUnicodeString(subject, "utf-8");
+    if (!mail->messageID(false)) {
+        //A globally unique messageId that doesn't leak the local hostname
+        const auto messageId = "<" + QUuid::createUuid().toString().mid(1, 36).remove('-') + "@kube>";
+        mail->messageID(true)->fromUnicodeString(messageId, "utf-8");
+    }
+    if (!mail->date(true)->dateTime().isValid()) {
+        mail->date(true)->setDateTime(QDateTime::currentDateTimeUtc());
+    }
+    mail->assemble();
+
+    std::unique_ptr<KMime::Content> bodyPart{[&] {
+        if (!attachments.isEmpty()) {
+            auto bodyPart = new KMime::Content;
+            bodyPart->contentType(true)->setMimeType("multipart/mixed");
+            bodyPart->contentType()->setBoundary(KMime::multiPartBoundary());
+            bodyPart->contentTransferEncoding()->setEncoding(KMime::Headers::CE7Bit);
+            bodyPart->setPreamble("This is a multi-part message in MIME format.\n");
+            bodyPart->addContent(createBodyPart(body, htmlBody));
+            for (const auto &attachment : attachments) {
+                bodyPart->addContent(createAttachmentPart(attachment.data, attachment.filename, attachment.isInline, attachment.mimeType, attachment.name));
+            }
+            return bodyPart;
+        } else {
+            return createBodyPart(body, htmlBody);
+        }
+    }()};
+    bodyPart->assemble();
+
+    QByteArray bodyData;
+    if (!signingKeys.empty() || !encryptionKeys.empty()) {
+        auto result = MailCrypto::processCrypto(std::move(bodyPart), signingKeys, encryptionKeys, attachedKey);
+        if (!result) {
+            qWarning() << "Crypto failed";
+            return {};
+        }
+        bodyData = result.value()->encodedContent();
+    } else {
+        if (!bodyPart->contentType(false)) {
+            bodyPart->contentType(true)->setMimeType("text/plain");
+            bodyPart->assemble();
+        }
+        bodyData = bodyPart->encodedContent();
+    }
+
+    KMime::Message::Ptr resultMessage(new KMime::Message);
+    resultMessage->setContent(mail->head() + bodyData);
+    resultMessage->parse(); // Not strictly necessary.
+    return resultMessage;
+}
