@@ -1,5 +1,6 @@
 /*
    Copyright (c) 2015 Sandro Knauß <sknauss@kde.org>
+   Copyright (c) 2017 Christian Mollekopf <mollekopf@kolabsys.com>
 
    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published by
@@ -21,43 +22,26 @@
 #include "mimetreeparser_debug.h"
 #include "cryptohelper.h"
 #include "objecttreeparser.h"
-#include "qgpgmejobexecutor.h"
-
-#include "cryptobodypartmemento.h"
-#include "decryptverifybodypartmemento.h"
-#include "verifydetachedbodypartmemento.h"
-#include "verifyopaquebodypartmemento.h"
 
 #include "utils.h"
 
 #include <KMime/Content>
 
-#include <QGpgME/DN>
-#include <QGpgME/Protocol>
-#include <QGpgME/ImportJob>
-#include <QGpgME/KeyListJob>
-#include <QGpgME/VerifyDetachedJob>
-#include <QGpgME/VerifyOpaqueJob>
-
-#include <gpgme++/key.h>
-#include <gpgme++/keylistresult.h>
-#include <gpgme.h>
-
-#include <KLocalizedString>
+#include <crypto.h>
 
 #include <QTextCodec>
-#include <sstream>
 
 using namespace MimeTreeParser;
+using namespace Crypto;
 
 //------MessagePart-----------------------
 MessagePart::MessagePart(ObjectTreeParser *otp, const QString &text, KMime::Content *node)
     : mText(text)
     , mOtp(otp)
-    , mNode(node) //only null for messagepartlist
     , mParentPart(nullptr)
-    , mRoot(false)
+    , mNode(node) //only null for messagepartlist
     , mError(NoError)
+    , mRoot(false)
 {
 }
 
@@ -251,6 +235,28 @@ void MessagePart::parseInternal(KMime::Content *node, bool onlyOneMimePart)
     }
 }
 
+void MessagePart::parseInternal(const QByteArray &data)
+{
+    auto tempNode = new KMime::Content();
+
+    const auto lfData = KMime::CRLFtoLF(data);
+    //We have to deal with both bodies and full parts. In inline encrypted/signed parts we can have nested parts,
+    //or just plain-text, and both ends up here. setContent defaults to setting only the header, so we have to avoid this.
+    if (lfData.contains("\n\n")) {
+        tempNode->setContent(lfData);
+    } else {
+        tempNode->setBody(lfData);
+    }
+    tempNode->parse();
+    bindLifetime(tempNode);
+
+    if (!tempNode->head().isEmpty()) {
+        tempNode->contentDescription()->from7BitString("temporary node");
+    }
+
+    parseInternal(tempNode);
+}
+
 QString MessagePart::renderInternalText() const
 {
     QString text;
@@ -382,9 +388,10 @@ void TextMessagePart::parseContent()
     const QString &fromAddress = mOtp->nodeHelper()->fromAsString(mNode);
     mSignatureState  = KMMsgNotSigned;
     mEncryptionState = KMMsgNotEncrypted;
-    const auto blocks = prepareMessageForDecryption(mNode->decodedContent());
+    auto body = mNode->decodedContent();
+    const auto blocks = prepareMessageForDecryption(body);
 
-    const auto cryptProto = QGpgME::openpgp();
+    const auto cryptProto = OpenPGP;
 
     if (!blocks.isEmpty()) {
 
@@ -548,53 +555,40 @@ QString MimeMessagePart::htmlContent() const
 
 //-----AlternativeMessagePart----------------------
 
-AlternativeMessagePart::AlternativeMessagePart(ObjectTreeParser *otp, KMime::Content *node, Util::HtmlMode preferredMode)
+AlternativeMessagePart::AlternativeMessagePart(ObjectTreeParser *otp, KMime::Content *node)
     : MessagePart(otp, QString(), node)
-    , mPreferredMode(preferredMode)
 {
-    KMime::Content *dataIcal = findTypeInDirectChilds(mNode, "text/calendar");
-    KMime::Content *dataHtml = findTypeInDirectChilds(mNode, "text/html");
-    KMime::Content *dataText = findTypeInDirectChilds(mNode, "text/plain");
+    if (auto dataIcal = findTypeInDirectChilds(mNode, "text/calendar")) {
+        mChildParts[Util::MultipartIcal] = MimeMessagePart::Ptr(new MimeMessagePart(mOtp, dataIcal, true));
+    }
 
-    if (!dataHtml) {
+    if (auto dataText = findTypeInDirectChilds(mNode, "text/plain")) {
+        mChildParts[Util::MultipartPlain] = MimeMessagePart::Ptr(new MimeMessagePart(mOtp, dataText, true));
+    }
+
+    if (auto dataHtml = findTypeInDirectChilds(mNode, "text/html")) {
+        mChildParts[Util::MultipartHtml] = MimeMessagePart::Ptr(new MimeMessagePart(mOtp, dataHtml, true));
+    } else {
         // If we didn't find the HTML part as the first child of the multipart/alternative, it might
         // be that this is a HTML message with images, and text/plain and multipart/related are the
         // immediate children of this multipart/alternative node.
         // In this case, the HTML node is a child of multipart/related.
-        dataHtml = findTypeInDirectChilds(mNode, "multipart/related");
-
-        // Still not found? Stupid apple mail actually puts the attachments inside of the
-        // multipart/alternative, which is wrong. Therefore we also have to look for multipart/mixed
-        // here.
-        // Do this only when prefering HTML mail, though, since otherwise the attachments are hidden
-        // when displaying plain text.
-        if (!dataHtml) {
-            dataHtml  = findTypeInDirectChilds(mNode, "multipart/mixed");
+        if (auto data = findTypeInDirectChilds(mNode, "multipart/related")) {
+            const auto parts = data->contents();
+            for (int i = 0; i < parts.size(); i++) {
+                const auto p = parts.at(i);
+                if (i == 0 ) {
+                    mChildParts[Util::MultipartHtml] = MimeMessagePart::Ptr(new MimeMessagePart(mOtp, p, true));
+                } else if (KMime::isAttachment(p)) {
+                    appendSubPart(MimeMessagePart::Ptr(new MimeMessagePart(otp, p, true)));
+                }
+            }
+        //Same for multipart/mixed
+        } else if (auto data  = findTypeInDirectChilds(mNode, "multipart/mixed")) {
+            //FIXME: This doesn't populate mChildParts but instead attaches subparts. That way we at least process all parts as we should.
+            parseInternal(data);
         }
     }
-
-    if (dataIcal) {
-        mChildNodes[Util::MultipartIcal] = dataIcal;
-    }
-
-    if (dataText) {
-        mChildNodes[Util::MultipartPlain] = dataText;
-    }
-
-    if (dataHtml) {
-        mChildNodes[Util::MultipartHtml] = dataHtml;
-    }
-
-    if (mChildNodes.isEmpty()) {
-        qCWarning(MIMETREEPARSER_LOG) << "no valid nodes";
-        return;
-    }
-
-    QMapIterator<Util::HtmlMode, KMime::Content *> i(mChildNodes);
-    while (i.hasNext()) {
-        i.next();
-        mChildParts[i.key()] = MimeMessagePart::Ptr(new MimeMessagePart(mOtp, i.value(), true));
-    }
 }
 
 AlternativeMessagePart::~AlternativeMessagePart()
@@ -602,11 +596,6 @@ AlternativeMessagePart::~AlternativeMessagePart()
 
 }
 
-Util::HtmlMode AlternativeMessagePart::preferredMode() const
-{
-    return mPreferredMode;
-}
-
 QList<Util::HtmlMode> AlternativeMessagePart::availableModes()
 {
     return mChildParts.keys();
@@ -641,9 +630,9 @@ QString AlternativeMessagePart::htmlContent() const
 
 //-----CertMessageBlock----------------------
 
-CertMessagePart::CertMessagePart(ObjectTreeParser *otp, KMime::Content *node, const QGpgME::Protocol *cryptoProto)
+CertMessagePart::CertMessagePart(ObjectTreeParser *otp, KMime::Content *node, const CryptoProtocol cryptoProto)
     : MessagePart(otp, QString(), node)
-    , mCryptoProto(cryptoProto)
+    , mProtocol(cryptoProto)
 {
     if (!mNode) {
         qCWarning(MIMETREEPARSER_LOG) << "not a valid node";
@@ -659,9 +648,7 @@ CertMessagePart::~CertMessagePart()
 void CertMessagePart::import()
 {
     const QByteArray certData = mNode->decodedContent();
-    QGpgME::ImportJob *import = mCryptoProto->importJob();
-    QGpgMEJobExecutor executor;
-    auto result = executor.exec(import, certData);
+    importKeys(mProtocol, certData);
 }
 
 QString CertMessagePart::text() const
@@ -672,19 +659,17 @@ QString CertMessagePart::text() const
 //-----SignedMessageBlock---------------------
 SignedMessagePart::SignedMessagePart(ObjectTreeParser *otp,
                                      const QString &text,
-                                     const QGpgME::Protocol *cryptoProto,
+                                     const CryptoProtocol cryptoProto,
                                      const QString &fromAddress,
                                      KMime::Content *node, KMime::Content *signedData)
     : MessagePart(otp, text, node)
-    , mCryptoProto(cryptoProto)
+    , mProtocol(cryptoProto)
     , mFromAddress(fromAddress)
     , mSignedData(signedData)
 {
     mMetaData.isSigned = true;
     mMetaData.isGoodSignature = false;
-    mMetaData.keyTrust = GpgME::Signature::Unknown;
-    mMetaData.status = i18n("Wrong Crypto Plug-In.");
-    mMetaData.status_code = GPGME_SIG_STAT_NONE;
+    mMetaData.status = tr("Wrong Crypto Plug-In.");
 }
 
 SignedMessagePart::~SignedMessagePart()
@@ -702,203 +687,71 @@ bool SignedMessagePart::isSigned() const
     return mMetaData.isSigned;
 }
 
-bool SignedMessagePart::okVerify(const QByteArray &data, const QByteArray &signature, KMime::Content *textNode)
+
+static QString prettifyDN(const char *uid)
 {
-    NodeHelper *nodeHelper = mOtp->nodeHelper();
+    // We used to use QGpgME::DN::prettyDN here. But I'm not sure what we actually need it for.
+    return QString::fromUtf8(uid);
+}
 
-    mMetaData.isSigned = false;
-    mMetaData.keyTrust = GpgME::Signature::Unknown;
-    mMetaData.status = i18n("Wrong Crypto Plug-In.");
-    mMetaData.status_code = GPGME_SIG_STAT_NONE;
+void SignedMessagePart::sigStatusToMetaData(const Signature &signature)
+{
+    mMetaData.isGoodSignature = signature.status == GPG_ERR_NO_ERROR;
+    // save extended signature status flags
+    auto summary = signature.summary;
+    mMetaData.keyMissing = summary & GPGME_SIGSUM_KEY_MISSING;
+    mMetaData.keyExpired = summary & GPGME_SIGSUM_KEY_EXPIRED;
+    mMetaData.keyRevoked = summary & GPGME_SIGSUM_KEY_REVOKED;
+    mMetaData.sigExpired = summary & GPGME_SIGSUM_SIG_EXPIRED;
+    mMetaData.crlMissing = summary & GPGME_SIGSUM_CRL_MISSING;
+    mMetaData.crlTooOld = summary & GPGME_SIGSUM_CRL_TOO_OLD;
 
-    const QByteArray mementoName = "verification";
-
-    //TODO for the async case remember the memento
-    CryptoBodyPartMemento *m = nullptr;
-    Q_ASSERT(!m || mCryptoProto); //No CryptoPlugin and having a bodyPartMemento -> there is something completely wrong
-
-    if (!m && mCryptoProto) {
-        if (!signature.isEmpty()) {
-            QGpgME::VerifyDetachedJob *job = mCryptoProto->verifyDetachedJob();
-            if (job) {
-                m = new VerifyDetachedBodyPartMemento(job, mCryptoProto->keyListJob(), signature, data);
-            }
+    Key key;
+    if (mMetaData.isGoodSignature) {
+        // Search for the key by its fingerprint so that we can check for trust etc.
+        const auto keys =  findKeys({signature.fingerprint});
+        if (keys.size() > 1) {
+            // Should not happen
+            qCDebug(MIMETREEPARSER_LOG) << "Oops: Found more then one Key for Fingerprint: " << signature.fingerprint;
+        }
+        if (keys.empty()) {
+            // Should not happen at this point
+            qCWarning(MIMETREEPARSER_LOG) << "Oops: Found no Key for Fingerprint: " << signature.fingerprint;
         } else {
-            QGpgME::VerifyOpaqueJob *job = mCryptoProto->verifyOpaqueJob();
-            if (job) {
-                m = new VerifyOpaqueBodyPartMemento(job, mCryptoProto->keyListJob(), data);
-            }
+            key = keys[0];
         }
-        if (m) {
-            // if (mOtp->allowAsync()) {
-            //     QObject::connect(m, &CryptoBodyPartMemento::update,
-            //                      nodeHelper, &NodeHelper::update);
-            //     // QObject::connect(m, SIGNAL(update(MimeTreeParser::UpdateMode)),
-            //     //                  _source->sourceObject(), SLOT(update(MimeTreeParser::UpdateMode)));
-
-            //     if (m->start()) {
-            //         mMetaData.inProgress = true;
-            //         mOtp->mHasPendingAsyncJobs = true;
-            //     }
-            //     //FIXME delete memento once done
-            // } else {
-                m->exec();
-            // }
-        }
-    //only relevant in async case
-    // } else if (m->isRunning()) {
-    //     mMetaData.inProgress = true;
-    //     mOtp->mHasPendingAsyncJobs = true;
-    // } else {
-    //     mMetaData.inProgress = false;
-    //     mOtp->mHasPendingAsyncJobs = false;
     }
 
-    if (m && !mMetaData.inProgress) {
-        if (!signature.isEmpty()) {
-            mVerifiedText = data;
+    mMetaData.keyId = key.keyId;
+    if (mMetaData.keyId.isEmpty()) {
+        mMetaData.keyId = signature.fingerprint;
+    }
+    mMetaData.keyIsTrusted = signature.validity == GPGME_VALIDITY_FULL || signature.validity == GPGME_VALIDITY_ULTIMATE;
+    if (!key.userIds.empty()) {
+        mMetaData.signer = prettifyDN(key.userIds[0].id);
+    }
+    for (const auto &userId : key.userIds) {
+        QString email = QString::fromUtf8(userId.email);
+        // ### work around gpgme 0.3.QString text() const Q_DECL_OVERRIDE;x / cryptplug bug where the
+        // ### email addresses are specified as angle-addr, not addr-spec:
+        if (email.startsWith(QLatin1Char('<')) && email.endsWith(QLatin1Char('>'))) {
+            email = email.mid(1, email.length() - 2);
+        }
+        if (!email.isEmpty()) {
+            mMetaData.signerMailAddresses.append(email);
         }
-        setVerificationResult(m, textNode);
     }
 
-    if (!m && !mMetaData.inProgress) {
-        QString errorMsg;
-        QString cryptPlugLibName;
-        QString cryptPlugDisplayName;
-        if (mCryptoProto) {
-            cryptPlugLibName = mCryptoProto->name();
-            cryptPlugDisplayName = mCryptoProto->displayName();
+    mMetaData.creationTime = signature.creationTime;
+    if (mMetaData.signer.isEmpty()) {
+        if (!key.userIds.empty()) {
+            mMetaData.signer = prettifyDN(key.userIds[0].name);
         }
-
-        if (!mCryptoProto) {
-            if (cryptPlugDisplayName.isEmpty()) {
-                errorMsg = i18n("No appropriate crypto plug-in was found.");
+        if (!mMetaData.signerMailAddresses.empty()) {
+            if (mMetaData.signer.isEmpty()) {
+                mMetaData.signer = mMetaData.signerMailAddresses.front();
             } else {
-                errorMsg = i18nc("%1 is either 'OpenPGP' or 'S/MIME'",
-                                 "No %1 plug-in was found.",
-                                 cryptPlugDisplayName);
-            }
-        } else {
-            errorMsg = i18n("Crypto plug-in \"%1\" cannot verify signatures.",
-                            cryptPlugLibName);
-        }
-        mMetaData.errorText = i18n("The message is signed, but the "
-                                   "validity of the signature cannot be "
-                                   "verified.<br />"
-                                   "Reason: %1",
-                                   errorMsg);
-    }
-    //TODO don't delete in async case
-    if (m) {
-        delete m;
-    }
-
-    return mMetaData.isSigned;
-}
-
-static int signatureToStatus(const GpgME::Signature &sig)
-{
-    switch (sig.status().code()) {
-    case GPG_ERR_NO_ERROR:
-        return GPGME_SIG_STAT_GOOD;
-    case GPG_ERR_BAD_SIGNATURE:
-        return GPGME_SIG_STAT_BAD;
-    case GPG_ERR_NO_PUBKEY:
-        return GPGME_SIG_STAT_NOKEY;
-    case GPG_ERR_NO_DATA:
-        return GPGME_SIG_STAT_NOSIG;
-    case GPG_ERR_SIG_EXPIRED:
-        return GPGME_SIG_STAT_GOOD_EXP;
-    case GPG_ERR_KEY_EXPIRED:
-        return GPGME_SIG_STAT_GOOD_EXPKEY;
-    default:
-        return GPGME_SIG_STAT_ERROR;
-    }
-}
-
-QString prettifyDN(const char *uid)
-{
-    return QGpgME::DN(uid).prettyDN();
-}
-
-void SignedMessagePart::sigStatusToMetaData()
-{
-    GpgME::Key key;
-    if (mMetaData.isSigned) {
-        GpgME::Signature signature = mSignatures.front();
-        mMetaData.status_code = signatureToStatus(signature);
-        mMetaData.isGoodSignature = mMetaData.status_code & GPGME_SIG_STAT_GOOD;
-        // save extended signature status flags
-        mMetaData.sigSummary = signature.summary();
-
-        if (mMetaData.isGoodSignature && !key.keyID()) {
-            // Search for the key by its fingerprint so that we can check for
-            // trust etc.
-            QGpgME::KeyListJob *job = mCryptoProto->keyListJob(false);    // local, no sigs
-            if (!job) {
-                qCDebug(MIMETREEPARSER_LOG) << "The Crypto backend does not support listing keys. ";
-            } else {
-                std::vector<GpgME::Key> found_keys;
-                // As we are local it is ok to make this synchronous
-                GpgME::KeyListResult res = job->exec(QStringList(QLatin1String(signature.fingerprint())), false, found_keys);
-                if (res.error()) {
-                    qCDebug(MIMETREEPARSER_LOG) << "Error while searching key for Fingerprint: " << signature.fingerprint();
-                }
-                if (found_keys.size() > 1) {
-                    // Should not Happen
-                    qCDebug(MIMETREEPARSER_LOG) << "Oops: Found more then one Key for Fingerprint: " << signature.fingerprint();
-                }
-                if (found_keys.size() != 1) {
-                    // Should not Happen at this point
-                    qCDebug(MIMETREEPARSER_LOG) << "Oops: Found no Key for Fingerprint: " << signature.fingerprint();
-                } else {
-                    key = found_keys[0];
-                }
-                delete job;
-            }
-        }
-
-        if (key.keyID()) {
-            mMetaData.keyId = key.keyID();
-        }
-        if (mMetaData.keyId.isEmpty()) {
-            mMetaData.keyId = signature.fingerprint();
-        }
-        mMetaData.keyTrust = signature.validity();
-        if (key.numUserIDs() > 0 && key.userID(0).id()) {
-            mMetaData.signer = prettifyDN(key.userID(0).id());
-        }
-        for (uint iMail = 0; iMail < key.numUserIDs(); ++iMail) {
-            // The following if /should/ always result in TRUE but we
-            // won't trust implicitely the plugin that gave us these data.
-            if (key.userID(iMail).email()) {
-                QString email = QString::fromUtf8(key.userID(iMail).email());
-                // ### work around gpgme 0.3.QString text() const Q_DECL_OVERRIDE;x / cryptplug bug where the
-                // ### email addresses are specified as angle-addr, not addr-spec:
-                if (email.startsWith(QLatin1Char('<')) && email.endsWith(QLatin1Char('>'))) {
-                    email = email.mid(1, email.length() - 2);
-                }
-                if (!email.isEmpty()) {
-                    mMetaData.signerMailAddresses.append(email);
-                }
-            }
-        }
-
-        if (signature.creationTime()) {
-            mMetaData.creationTime.setTime_t(signature.creationTime());
-        } else {
-            mMetaData.creationTime = QDateTime();
-        }
-        if (mMetaData.signer.isEmpty()) {
-            if (key.numUserIDs() > 0 && key.userID(0).name()) {
-                mMetaData.signer = prettifyDN(key.userID(0).name());
-            }
-            if (!mMetaData.signerMailAddresses.empty()) {
-                if (mMetaData.signer.isEmpty()) {
-                    mMetaData.signer = mMetaData.signerMailAddresses.front();
-                } else {
-                    mMetaData.signer += QLatin1String(" <") + mMetaData.signerMailAddresses.front() + QLatin1Char('>');
-                }
+                mMetaData.signer += QLatin1String(" <") + mMetaData.signerMailAddresses.front() + QLatin1Char('>');
             }
         }
     }
@@ -908,78 +761,58 @@ void SignedMessagePart::startVerification()
 {
     if (mSignedData) {
         const QByteArray cleartext = KMime::LFtoCRLF(mSignedData->encodedContent());
-        const QTextCodec *aCodec(mOtp->codecFor(mSignedData));
 
         //The case for pkcs7
         if (mNode == mSignedData) {
             startVerificationDetached(cleartext, nullptr, {});
         } else {
-            startVerificationDetached(cleartext, mSignedData, mNode->decodedContent());
+            if (mNode) {
+                startVerificationDetached(cleartext, mSignedData, mNode->decodedContent());
+            } else { //The case for clearsigned above
+                startVerificationDetached(cleartext, nullptr, {});
+            }
         }
     }
 }
 
-void SignedMessagePart::startVerification(const QByteArray &text, const QTextCodec *aCodec)
-{
-    startVerificationDetached(text, nullptr, QByteArray());
-
-    if (!mNode && mMetaData.isSigned) {
-        setText(aCodec->toUnicode(mVerifiedText));
-    }
-}
-
 void SignedMessagePart::startVerificationDetached(const QByteArray &text, KMime::Content *textNode, const QByteArray &signature)
 {
     mMetaData.isEncrypted = false;
     mMetaData.isDecryptable = false;
 
     if (textNode) {
-        parseInternal(textNode, false);
+        parseInternal(textNode);
     }
 
-    okVerify(text, signature, textNode);
+    mMetaData.isSigned = false;
+    mMetaData.status = tr("Wrong Crypto Plug-In.");
+
+    if (!signature.isEmpty()) {
+        setVerificationResult(verifyDetachedSignature(mProtocol, signature, text), false, text);
+    } else {
+        QByteArray outdata;
+        setVerificationResult(verifyOpaqueSignature(mProtocol, text, outdata), false, outdata);
+    }
 
     if (!mMetaData.isSigned) {
         mMetaData.creationTime = QDateTime();
     }
 }
 
-void SignedMessagePart::setVerificationResult(const CryptoBodyPartMemento *m, KMime::Content *textNode)
+void SignedMessagePart::setVerificationResult(const VerificationResult &result, bool parseText, const QByteArray &plainText)
 {
-    if (const auto vm = dynamic_cast<const VerifyDetachedBodyPartMemento *>(m)) {
-        mSignatures = vm->verifyResult().signatures();
-    }
-    if (const auto vm = dynamic_cast<const VerifyOpaqueBodyPartMemento *>(m)) {
-        mVerifiedText = vm->plainText();
-        mSignatures = vm->verifyResult().signatures();
-    }
-    if (const auto vm = dynamic_cast<const DecryptVerifyBodyPartMemento *>(m)) {
-        mVerifiedText = vm->plainText();
-        mSignatures = vm->verifyResult().signatures();
-    }
-    mMetaData.auditLogError = m->auditLogError();
-    mMetaData.auditLog = m->auditLogAsHtml();
-    mMetaData.isSigned = !mSignatures.empty();
+    auto signatures = result.signatures;
+    // FIXME
+    // mMetaData.auditLogError = result.error;
+    if (!signatures.empty()) {
+        mMetaData.isSigned = true;
+        sigStatusToMetaData(signatures.front());
+        if (mNode && parseText) {
+            mOtp->mNodeHelper->setPartMetaData(mNode, mMetaData);
+        }
+        if (!plainText.isEmpty() && parseText) {
+            parseInternal(plainText);
 
-    if (mMetaData.isSigned) {
-        sigStatusToMetaData();
-        if (mNode) {
-            if (!textNode) {
-                mOtp->mNodeHelper->setPartMetaData(mNode, mMetaData);
-
-                if (!mVerifiedText.isEmpty()) {
-                    auto tempNode = new KMime::Content();
-                    tempNode->setContent(KMime::CRLFtoLF(mVerifiedText.constData()));
-                    tempNode->parse();
-                    bindLifetime(tempNode);
-
-                    if (!tempNode->head().isEmpty()) {
-                        tempNode->contentDescription()->from7BitString("signed data");
-                    }
-
-                    parseInternal(tempNode, false);
-                }
-            }
         }
     }
 }
@@ -1005,11 +838,11 @@ QString SignedMessagePart::htmlContent() const
 //-----CryptMessageBlock---------------------
 EncryptedMessagePart::EncryptedMessagePart(ObjectTreeParser *otp,
         const QString &text,
-        const QGpgME::Protocol *cryptoProto,
+        const CryptoProtocol cryptoProto,
         const QString &fromAddress,
         KMime::Content *node, KMime::Content *encryptedNode)
     : MessagePart(otp, text, node)
-    , mCryptoProto(cryptoProto)
+    , mProtocol(cryptoProto)
     , mFromAddress(fromAddress)
     , mEncryptedNode(encryptedNode)
 {
@@ -1017,9 +850,7 @@ EncryptedMessagePart::EncryptedMessagePart(ObjectTreeParser *otp,
     mMetaData.isGoodSignature = false;
     mMetaData.isEncrypted = false;
     mMetaData.isDecryptable = false;
-    mMetaData.keyTrust = GpgME::Signature::Unknown;
-    mMetaData.status = i18n("Wrong Crypto Plug-In.");
-    mMetaData.status_code = GPGME_SIG_STAT_NONE;
+    mMetaData.status = tr("Wrong Crypto Plug-In.");
 }
 
 EncryptedMessagePart::~EncryptedMessagePart()
@@ -1051,17 +882,18 @@ void EncryptedMessagePart::startDecryption(const QByteArray &text, const QTextCo
 
     startDecryption(content);
 
-    auto code = aCodec ? aCodec : mOtp->codecFor(mNode);
-    if (!mMetaData.inProgress && mMetaData.isDecryptable) {
+    if (mMetaData.isDecryptable) {
+        const auto codec = aCodec ? aCodec : mOtp->codecFor(mNode);
+        const auto decoded = codec->toUnicode(mDecryptedData);
         if (hasSubParts()) {
             auto _mp = (subParts()[0]).dynamicCast<SignedMessagePart>();
             if (_mp) {
-                _mp->setText(aCodec->toUnicode(mDecryptedData));
+                _mp->setText(decoded);
             } else {
-                setText(aCodec->toUnicode(mDecryptedData));
+                setText(decoded);
             }
         } else {
-            setText(aCodec->toUnicode(mDecryptedData));
+            setText(decoded);
         }
     }
 }
@@ -1069,85 +901,68 @@ void EncryptedMessagePart::startDecryption(const QByteArray &text, const QTextCo
 bool EncryptedMessagePart::okDecryptMIME(KMime::Content &data)
 {
     mError = NoError;
-    auto passphraseError = false;
-    auto noSecKey = false;
-    mMetaData.inProgress = false;
     mMetaData.errorText.clear();
-    mMetaData.auditLogError = GpgME::Error();
+    //FIXME
+    // mMetaData.auditLogError = GpgME::Error();
     mMetaData.auditLog.clear();
-    NodeHelper *nodeHelper = mOtp->nodeHelper();
-
-    if (!mCryptoProto) {
-        mError = UnknownError;
-        mMetaData.errorText = i18n("No appropriate crypto plug-in was found.");
-        return false;
-    }
-
-    QGpgME::DecryptVerifyJob *job = mCryptoProto->decryptVerifyJob();
-    if (!job) {
-        mError = UnknownError;
-        mMetaData.errorText = i18n("Crypto plug-in \"%1\" cannot decrypt messages.", mCryptoProto->name());
-        return false;
-    }
 
     const QByteArray ciphertext = data.decodedContent();
-    auto m = QSharedPointer<DecryptVerifyBodyPartMemento>::create(job, ciphertext);
-    m->exec();
+    QByteArray plainText;
+    DecryptionResult decryptResult;
+    VerificationResult verifyResult;
+    std::tie(decryptResult, verifyResult) = decryptAndVerify(mProtocol, ciphertext, plainText);
+    mMetaData.isSigned = verifyResult.signatures.size() > 0;
 
-    const QByteArray &plainText = m->plainText();
-    const GpgME::DecryptionResult &decryptResult = m->decryptResult();
-    const GpgME::VerificationResult &verifyResult = m->verifyResult();
-    mMetaData.isSigned = verifyResult.signatures().size() > 0;
-
-    if (verifyResult.signatures().size() > 0) {
+    if (verifyResult.signatures.size() > 0) {
         //We simply attach a signed message part to indicate that this content is also signed
-        auto subPart = SignedMessagePart::Ptr(new SignedMessagePart(mOtp, QString::fromUtf8(plainText), mCryptoProto, mFromAddress, nullptr, nullptr));
-        subPart->setVerificationResult(m.data(), nullptr);
+        auto subPart = SignedMessagePart::Ptr(new SignedMessagePart(mOtp, QString::fromUtf8(plainText), mProtocol, mFromAddress, nullptr, nullptr));
+        subPart->setVerificationResult(verifyResult, true, plainText);
         appendSubPart(subPart);
     }
 
-    mDecryptRecipients = decryptResult.recipients();
-    auto decryptionOk = !decryptResult.error();
-
-    if (!decryptionOk && mMetaData.isSigned) {
+    if (decryptResult.error && mMetaData.isSigned) {
         //Only a signed part
         mMetaData.isEncrypted = false;
         mDecryptedData = plainText;
         return true;
     }
 
-    passphraseError =  decryptResult.error().isCanceled() || decryptResult.error().code() == GPG_ERR_NO_SECKEY;
-    mMetaData.isEncrypted = decryptResult.error().code() != GPG_ERR_NO_DATA;
-    mMetaData.errorText = QString::fromLocal8Bit(decryptResult.error().asString());
-    if (mMetaData.isEncrypted && decryptResult.numRecipients() > 0) {
-        mMetaData.keyId = decryptResult.recipient(0).keyID();
+    if (mMetaData.isEncrypted && decryptResult.recipients.size()) {
+        mMetaData.keyId = decryptResult.recipients.at(0).keyId;
     }
 
-    if (decryptionOk) {
+    if (!decryptResult.error) {
         mDecryptedData = plainText;
         setText(QString::fromUtf8(mDecryptedData.constData()));
     } else {
-        noSecKey = true;
-        foreach (const GpgME::DecryptionResult::Recipient &recipient, decryptResult.recipients()) {
-            noSecKey &= (recipient.status().code() == GPG_ERR_NO_SECKEY);
-        }
-        if (!passphraseError && !noSecKey) {          // GpgME do not detect passphrase error correctly
+        const auto errorCode = decryptResult.error.errorCode();
+        mMetaData.isEncrypted = errorCode != GPG_ERR_NO_DATA;
+        qWarning() << "Failed to decrypt : " << decryptResult.error;
+
+        const bool noSecretKeyAvilable = [&] {
+            foreach (const auto &recipient, decryptResult.recipients) {
+                if (!(recipient.status.errorCode() == GPG_ERR_NO_SECKEY)) {
+                    return false;
+                }
+            }
+            return true;
+        }();
+        bool passphraseError = errorCode == GPG_ERR_CANCELED || errorCode == GPG_ERR_NO_SECKEY;
+        //We only get a decryption failed error when we enter the wrong passphrase....
+        if (!passphraseError && !noSecretKeyAvilable) {
             passphraseError = true;
         }
-        std::stringstream ss;
-        ss << decryptResult << '\n' << verifyResult;
-        qWarning() << "Decryption failed: " << ss.str().c_str();
 
-        if(noSecKey) {
+        if(noSecretKeyAvilable) {
             mError = NoKeyError;
-            mMetaData.errorText = i18n("Crypto plug-in \"%1\" could not decrypt the data. ", mCryptoProto->name())
-                                  + i18n("No key found for recepients.");
+            mMetaData.errorText = tr("Could not decrypt the data. ") + tr("No key found for recepients.");
         } else if (passphraseError) {
             mError = PassphraseError;
+            // mMetaData.errorText = QString::fromLocal8Bit(decryptResult.error().asString());
         } else {
             mError = UnknownError;
-            mMetaData.errorText = i18n("Crypto plug-in \"%1\" could not decrypt the data. ", mCryptoProto->name())
-                                  + i18n("Error: %1", mMetaData.errorText);
+            mMetaData.errorText = tr("Could not decrypt the data. ");
+            //                         + tr("Error: %1").arg(QString::fromLocal8Bit(decryptResult.error().asString()));
         }
         return false;
     }
@@ -1157,7 +972,6 @@ bool EncryptedMessagePart::okDecryptMIME(KMime::Content &data)
 
 void EncryptedMessagePart::startDecryption(KMime::Content *data)
 {
-
     if (!data) {
         data = mEncryptedNode;
         if (!data) {
@@ -1167,12 +981,7 @@ void EncryptedMessagePart::startDecryption(KMime::Content *data)
 
     mMetaData.isEncrypted = true;
 
-    bool bOkDecrypt = okDecryptMIME(*data);
-
-    if (mMetaData.inProgress) {
-        return;
-    }
-    mMetaData.isDecryptable = bOkDecrypt;
+    mMetaData.isDecryptable = okDecryptMIME(*data);
 
     if (!mMetaData.isDecryptable) {
         setText(QString::fromUtf8(mDecryptedData.constData()));
@@ -1184,16 +993,7 @@ void EncryptedMessagePart::startDecryption(KMime::Content *data)
 
     if (mNode && !mMetaData.isSigned) {
         mOtp->mNodeHelper->setPartMetaData(mNode, mMetaData);
-        auto tempNode = new KMime::Content();
-        tempNode->setContent(KMime::CRLFtoLF(mDecryptedData.constData()));
-        tempNode->parse();
-        bindLifetime(tempNode);
-
-        if (!tempNode->head().isEmpty()) {
-            tempNode->contentDescription()->from7BitString("encrypted data");
-        }
-
-        parseInternal(tempNode, false);
+        parseInternal(mDecryptedData);
     }
 }
 
@@ -1244,7 +1044,7 @@ EncapsulatedRfc822MessagePart::EncapsulatedRfc822MessagePart(ObjectTreeParser *o
         return;
     }
 
-    parseInternal(message.data(), false);
+    parseInternal(message.data());
 }
 
 EncapsulatedRfc822MessagePart::~EncapsulatedRfc822MessagePart()
@@ -1272,3 +1072,13 @@ QDateTime EncapsulatedRfc822MessagePart::date() const
     }
     return {};
 }
+
+HeadersPart::HeadersPart(ObjectTreeParser *otp, KMime::Content *node)
+    : MessagePart(otp, QString(), node)
+{
+}
+
+HeadersPart::~HeadersPart()
+{
+
+}
