@@ -32,7 +32,7 @@ namespace MimeTreeParser
      found in 'cntDesc'.
   2. Parse the 'node' to display the content.
 */
-MimeMessagePart::Ptr createAndParseTempNode(Interface::BodyPart &part, KMime::Content *parentNode, const char *content, const char *cntDesc);
+MimeMessagePart::Ptr createAndParseTempNode(Interface::BodyPart &part, const char *content, const char *cntDesc);
 
 KMime::Content *findTypeInDirectChilds(KMime::Content *content, const QByteArray &mimeType);
 
