@@ -24,6 +24,8 @@
 #include <QDir>
 #include <QUrl>
 
+#include "keyring.h"
+
 using namespace Sink;
 using namespace Sink::ApplicationDomain;
 
@@ -54,17 +56,16 @@ void AccountSettings::setAccountIdentifier(const QByteArray &id)
     mName = QString();
     mImapServer = QString();
     mImapUsername = QString();
-    mImapPassword = QString();
     mSmtpServer = QString();
     mSmtpUsername = QString();
-    mSmtpPassword = QString();
     mCardDavServer = QString();
     mCardDavUsername = QString();
-    mCardDavPassword = QString();
+    mPath = QString();
     emit changed();
     emit imapResourceChanged();
     emit smtpResourceChanged();
     emit cardDavResourceChanged();
+    emit pathChanged();
 
     load();
 
@@ -156,9 +157,9 @@ void AccountSettings::saveAccount()
             .onError([](const KAsync::Error &error) {
                 qWarning() << "Error while creating account: " << error.errorMessage;;
             })
-            .exec();
+            .exec().waitForFinished();
     } else {
-        qDebug() << "Saving account " << mAccountIdentifier << mMailtransportIdentifier;
+        qDebug() << "Saving account " << mAccountIdentifier;
         Q_ASSERT(!mAccountIdentifier.isEmpty());
         SinkAccount account(mAccountIdentifier);
         account.setAccountType(mAccountType);
@@ -169,7 +170,7 @@ void AccountSettings::saveAccount()
             .onError([](const KAsync::Error &error) {
                 qWarning() << "Error while creating account: " << error.errorMessage;;
             })
-            .exec();
+            .exec().waitForFinished();
     }
 }
 
@@ -182,50 +183,47 @@ void AccountSettings::loadAccount()
             mIcon = account.getIcon();
             mName = account.getName();
             emit changed();
-        }).exec();
+        }).onError([](const KAsync::Error &error) {
+            qWarning() << "Failed to load the account: " << error.errorMessage;
+        }).exec().waitForFinished();
 }
 
 void AccountSettings::loadImapResource()
 {
-    Store::fetchOne<SinkResource>(Query().filter<SinkResource::Account>(mAccountIdentifier).containsFilter<SinkResource::Capabilities>(ResourceCapabilities::Mail::storage))
+    Store::fetchOne<SinkResource>(Query().filter<SinkResource::Account>(mAccountIdentifier).filter<SinkResource::ResourceType>("sink.imap"))
         .then([this](const SinkResource &resource) {
             mImapIdentifier = resource.identifier();
             mImapServer = resource.getProperty("server").toString();
             mImapUsername = resource.getProperty("username").toString();
-            mImapPassword = resource.getProperty("password").toString();
             emit imapResourceChanged();
         }).onError([](const KAsync::Error &error) {
-            qWarning() << "Failed to find the imap resource: " << error.errorMessage;
-        }).exec();
+            qWarning() << "Failed to load the imap resource: " << error.errorMessage;
+        }).exec().waitForFinished();
 }
 
 void AccountSettings::loadMaildirResource()
 {
-    Store::fetchOne<SinkResource>(Query().filter<SinkResource::Account>(mAccountIdentifier).containsFilter<SinkResource::Capabilities>(ResourceCapabilities::Mail::storage))
+    Store::fetchOne<SinkResource>(Query().filter<SinkResource::Account>(mAccountIdentifier).filter<SinkResource::ResourceType>("sink.maildir"))
         .then([this](const SinkResource &resource) {
             mMaildirIdentifier = resource.identifier();
-            auto path = resource.getProperty("path").toString();
-            if (mPath != path) {
-                mPath = path;
-                emit pathChanged();
-            }
+            mPath = resource.getProperty("path").toString();
+            emit pathChanged();
         }).onError([](const KAsync::Error &error) {
-            SinkWarning() << "Failed to find the maildir resource: " << error.errorMessage;
-        }).exec();
+            SinkWarning() << "Failed to load the maildir resource: " << error.errorMessage;
+        }).exec().waitForFinished();
 }
 
 void AccountSettings::loadMailtransportResource()
 {
-    Store::fetchOne<SinkResource>(Query().filter<SinkResource::Account>(mAccountIdentifier).containsFilter<SinkResource::Capabilities>(ResourceCapabilities::Mail::transport))
+    Store::fetchOne<SinkResource>(Query().filter<SinkResource::Account>(mAccountIdentifier).filter<SinkResource::ResourceType>("sink.mailtransport"))
         .then([this](const SinkResource &resource) {
             mMailtransportIdentifier = resource.identifier();
             mSmtpServer = resource.getProperty("server").toString();
             mSmtpUsername = resource.getProperty("username").toString();
-            mSmtpPassword = resource.getProperty("password").toString();
             emit smtpResourceChanged();
         }).onError([](const KAsync::Error &error) {
-            SinkWarning() << "Failed to find the smtp resource: " << error.errorMessage;
-        }).exec();
+            SinkWarning() << "Failed to load the smtp resource: " << error.errorMessage;
+        }).exec().waitForFinished();
 }
 
 void AccountSettings::loadIdentity()
@@ -238,22 +236,21 @@ void AccountSettings::loadIdentity()
             mEmailAddress = identity.getAddress();
             emit identityChanged();
         }).onError([](const KAsync::Error &error) {
-            SinkWarning() << "Failed to find the identity resource: " << error.errorMessage;
-        }).exec();
+            SinkWarning() << "Failed to load the identity: " << error.errorMessage;
+        }).exec().waitForFinished();
 }
 
 void AccountSettings::loadCardDavResource()
 {
-    Store::fetchOne<SinkResource>(Query().filter<SinkResource::Account>(mAccountIdentifier).containsFilter<SinkResource::Capabilities>(ResourceCapabilities::Contact::storage))
+    Store::fetchOne<SinkResource>(Query().filter<SinkResource::Account>(mAccountIdentifier).filter<SinkResource::ResourceType>("sink.carddav"))
         .then([this](const SinkResource &resource) {
             mCardDavIdentifier = resource.identifier();
             mCardDavServer = resource.getProperty("server").toString();
             mCardDavUsername = resource.getProperty("username").toString();
-            mCardDavPassword = resource.getProperty("password").toString();
             emit cardDavResourceChanged();
         }).onError([](const KAsync::Error &error) {
-            qWarning() << "Failed to find the CardDAV resource: " << error.errorMessage;
-        }).exec();
+            qWarning() << "Failed to load the CardDAV resource: " << error.errorMessage;
+        }).exec().waitForFinished();
 }
 
 
@@ -269,7 +266,7 @@ static QByteArray saveResource(const QByteArray &accountIdentifier, const QByteA
             .onError([](const KAsync::Error &error) {
                 SinkWarning() << "Error while modifying resource: " << error.errorMessage;
             })
-            .exec();
+            .exec().waitForFinished();
     } else {
         auto resource = ResourceType::create(accountIdentifier);
         auto newIdentifier = resource.identifier();
@@ -280,7 +277,7 @@ static QByteArray saveResource(const QByteArray &accountIdentifier, const QByteA
             .onError([](const KAsync::Error &error) {
                 SinkWarning() << "Error while creating resource: " << error.errorMessage;
             })
-            .exec();
+            .exec().waitForFinished();
         return newIdentifier;
     }
     return identifier;
@@ -290,8 +287,7 @@ void AccountSettings::saveImapResource()
 {
     mImapIdentifier = saveResource<ImapResource>(mAccountIdentifier, mImapIdentifier, {
             {"server", mImapServer},
-            {"username", mImapUsername},
-            {"password", mImapPassword},
+            {"username", mImapUsername}
         });
 }
 
@@ -299,8 +295,7 @@ void AccountSettings::saveCardDavResource()
 {
     mCardDavIdentifier = saveResource<CardDavResource>(mAccountIdentifier, mCardDavIdentifier, {
             {"server", mCardDavServer},
-            {"username", mCardDavUsername},
-            {"password", mCardDavPassword},
+            {"username", mCardDavUsername}
         });
 }
 
@@ -315,22 +310,34 @@ void AccountSettings::saveMailtransportResource()
 {
     mMailtransportIdentifier = saveResource<MailtransportResource>(mAccountIdentifier, mMailtransportIdentifier, {
             {"server", mSmtpServer},
-            {"username", mSmtpUsername},
-            {"password", mSmtpPassword},
+            {"username", mSmtpUsername}
         });
 }
 
+void AccountSettings::login(const QVariantMap &secrets)
+{
+    auto accountSecret = secrets.value("accountSecret").toString();
+    Store::fetchAll<SinkResource>(Query().filter<SinkResource::Account>(mAccountIdentifier))
+        .then([=](const QList<SinkResource::Ptr> &resources) {
+            for (const auto &resource : resources) {
+                Kube::AccountKeyring{mAccountIdentifier}.storePassword(resource->identifier(), accountSecret);
+            }
+        }).onError([](const KAsync::Error &error) {
+            qWarning() << "Failed to load any account resources resource: " << error;
+        }).exec();
+}
+
 void AccountSettings::saveIdentity()
 {
     if (!mIdentityIdentifier.isEmpty()) {
-        Identity identity(mMailtransportIdentifier);
+        Identity identity(mIdentityIdentifier);
         identity.setName(mUsername);
         identity.setAddress(mEmailAddress);
         Store::modify(identity)
         .onError([](const KAsync::Error &error) {
             SinkWarning() << "Error while modifying identity: " << error.errorMessage;
         })
-        .exec();
+        .exec().waitForFinished();
     } else {
         auto identity = ApplicationDomainType::createEntity<Identity>();
         mIdentityIdentifier = identity.identifier();
@@ -341,7 +348,7 @@ void AccountSettings::saveIdentity()
         .onError([](const KAsync::Error &error) {
             SinkWarning() << "Error while creating identity: " << error.errorMessage;
         })
-        .exec();
+        .exec().waitForFinished();
     }
 }
 
@@ -355,7 +362,7 @@ void AccountSettings::removeResource(const QByteArray &identifier)
         .onError([](const KAsync::Error &error) {
             SinkWarning() << "Error while removing resource: " << error.errorMessage;
         })
-        .exec();
+        .exec().waitForFinished();
     }
 }
 
@@ -369,7 +376,7 @@ void AccountSettings::removeAccount()
         .onError([](const KAsync::Error &error) {
             SinkWarning() << "Error while removing account: " << error.errorMessage;
         })
-        .exec();
+        .exec().waitForFinished();
     }
 }
 
@@ -383,7 +390,7 @@ void AccountSettings::removeIdentity()
         .onError([](const KAsync::Error &error) {
             SinkWarning() << "Error while removing identity: " << error.errorMessage;
         })
-        .exec();
+        .exec().waitForFinished();
     }
 }
 
