@@ -29,5 +29,6 @@ class FrameworkPlugin : public QQmlExtensionPlugin
     Q_PLUGIN_METADATA(IID "org.qt-project.Qt.QQmlExtensionInterface")
 
 public:
-    virtual void registerTypes(const char *uri);
+    void registerTypes(const char *uri) Q_DECL_OVERRIDE;
+    void initializeEngine(QQmlEngine *engine, const char *uri) Q_DECL_OVERRIDE;
 };
