@@ -13,7 +13,9 @@ wantedIcons = [
     "document-decrypt.svg",
     "document-edit.svg",
     "document-encrypt.svg",
+    "view-certificate-import.svg",
     "document-save.svg",
+    "document-sign.svg",
     "edit-delete.svg",
     "edit-find.svg",
     "edit-undo.svg",
@@ -22,7 +24,9 @@ wantedIcons = [
     "im-user.svg",
     "mail-mark-important.svg",
     "mail-mark-unread-new.svg",
+    "mail-mark-read.svg",
     "mail-reply-sender.svg",
+    "mail-forward.svg",
     "mail-folder-outbox.svg",
     "network-disconnect.svg",
     "view-refresh.svg",
@@ -31,7 +35,18 @@ wantedIcons = [
     "go-previous.svg",
     "mail-message.svg",
     "list-add.svg",
-    "list-remove.svg"
+    "list-remove.svg",
+    "checkbox.svg",
+    "edit-copy.svg",
+    "password-show-on.svg",
+    "password-show-off.svg",
+    "format-text-bold-symbolic.svg",
+    "format-text-italic-symbolic.svg",
+    "format-text-underline-symbolic.svg",
+    "documentinfo.svg",
+    "group.svg",
+    "mail-task.svg",
+    "question.svg"
 ]
 
 def ensure_dir(file_path):
